@@ -3,7 +3,8 @@ function (sl.functions, sl.names, sl.mins, sl.maxs, sl.deltas,
     sl.defaults, but.functions, but.names, no, set.no.value, 
     obj.name, obj.value, reset.function, title, prompt=FALSE) 
 { # pwolf 080310 080603
-  if (!exists("slider.env")) slider.env <<- new.env(parent=.GlobalEnv)
+  slider.env<-"1"; rm("slider.env")
+  if (!exists("slider.env")) slider<-slider.env<<-new.env(parent=.GlobalEnv)
   if (!missing(no)) 
           return(as.numeric(tclvalue(get(paste("slider", no, sep = ""), 
               env = slider.env))))
